"""Processing of site.toml file."""
import toml

from .exceptions import InvalidConfigError


class Config:
    def __init__(self, data: str = "") -> None:
        self._config = toml.loads(data)

        try:
            self.name = self._config["name"]
            self.slogan = self._config.get("slogan", "A website!")
            self.preferences = self._config.get("preferences", {})
            self.plugins = self._config.get("plugins", [])
        except KeyError as e:
            raise InvalidConfigError(f"Missing item in config: {e.args[0]!r}")

    @property
    def raw(self) -> dict:
        """Return the raw config."""
        return self._config



import os
import shutil
from http.server import SimpleHTTPRequestHandler
from socketserver import TCPServer

import markdown
import frontmatter
from jinja2 import Environment, FileSystemLoader
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
from .config import Config
from .utils import (
    import_mod_from_path,
    out_name,
    special_file,
    find_template,
)


class DevFSHandler(FileSystemEventHandler):
    def __init__(self, app) -> None:
        super().__init__()
        self.app = app

    def on_any_event(self, event):
        self.app.build("dev.tmp")


class DevHTTPHandler(SimpleHTTPRequestHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, directory="dev.tmp", **kwargs)


class Site:
    def __init__(self, path: os.PathLike = "") -> None:
        self.path = path
        self.src_path = os.path.join(self.path, "src")
        self.jenv = Environment(loader=FileSystemLoader(self.src_path), autoescape=True)

        with open(os.path.join(path, "site.toml"), "r") as f:
            self.config = Config(f.read())

    def dev(self) -> None:
        self.build("dev.tmp")

        # Watch for changes
        observer = Observer()
        observer.schedule(DevFSHandler(self), self.path, recursive=True)
        observer.start()

        # Start HTTP server
        with TCPServer(("", 3000), DevHTTPHandler) as httpd:
            print("Serving at port", 3000)
            httpd.serve_forever()

        observer.join()

    def build(self, output: os.PathLike = "") -> None:
        """Build the site."""
        # Make temp output directory
        tmp_output = f"{output}.tmp"

        if os.path.exists(tmp_output):
            shutil.rmtree(tmp_output)

        # Copy static files
        shutil.copytree(os.path.join(self.path, "static"), os.path.join(tmp_output))

        # Generate pages
        for file in [
            os.path.relpath(os.path.join(path, name), start=self.src_path)
            for path, _, files in os.walk(self.src_path)
            for name in files
        ]:
            # Ignore special files (e.g. _base.jinja)
            if special_file(file):
                continue

            out_loc = os.path.join(tmp_output, out_name(file))

            # Make directory if it doesn't exist
            os.makedirs(os.path.dirname(out_loc), exist_ok=True)

            generated_page = self.generate_page(file)

            if not generated_page:
                continue

            with open(out_loc, "w") as f:
                f.write(generated_page)

        # Finish up

        # For all html files not named index.html, make it's own folder
        for file in [
            os.path.join(path, name)
            for path, _, files in os.walk(tmp_output)
            for name in files
        ]:
            if (
                file.endswith(".html")
                and not os.path.split(file)[-1].strip() == "index.html"
            ):
                if os.path.exists(
                    os.path.join(file.replace(".html", ""), "index.html")
                ):
                    continue

                os.makedirs(os.path.join(file.replace(".html", "")), exist_ok=True)
                shutil.copy(
                    os.path.join(file),
                    os.path.join(file.replace(".html", ""), "index.html"),
                )

        # Move temp output to output dir
        if os.path.exists(output):
            shutil.rmtree(output)

        shutil.move(tmp_output, output)

    def generate_page(self, loc: str) -> str:
        """Generate a page."""
        default_vars = {
            "site": {
                "name": self.config.name,
                "slogan": self.config.slogan,
            },
            "page": {
                "location": loc,
                "params": {},
            },
        }

        if loc.endswith(".jinja") or loc.endswith(".html"):
            # Get props from Python file, if exists
            py_loc = os.path.join(self.src_path, f"{'.'.join(loc.split('.')[:-1])}.py")

            props = None

            if os.path.exists(py_loc):
                mod = import_mod_from_path(py_loc)

                if "props" in dir(mod):
                    props = mod.props

            # Render template
            return self.jenv.get_template(loc).render(
                **default_vars,
                props=props,
            )
        elif loc.endswith(".md"):
            # Get metadata and content from frontmatter
            with open(os.path.join(self.src_path, loc), "r") as f:
                data = frontmatter.loads(f.read())

            # Render markdown using template
            base_template_loc = os.path.relpath(
                find_template(
                    self.src_path, os.path.join(self.src_path, loc), markdown=True
                ),
                start=self.src_path,
            )

            template = """
            {% extends _md.base %}
            {% block content %}
                {{ _md.content|safe }}
            {% endblock %}
            """

            # Add markdown content to variables
            default_vars["page"].update(
                {"metadata": data.metadata, "content": data.content}
            )

            # Render markdown
            rendered = markdown.markdown(data.content)

            # Render template
            return self.jenv.from_string(template).render(
                **default_vars, _md={"base": base_template_loc, "content": rendered}
            )
        elif loc.endswith(".py"):
            return None
        else:
            with open(os.path.join(self.src_path, loc), "r") as f:
                return f.read()



from typing import Optional
import os
import importlib.util


def import_mod_from_path(loc: os.PathLike) -> object:
    """Loads a module from a path and returns it."""
    spec = importlib.util.spec_from_file_location(
        f"crouton.__dyn__.{'.'.join(os.path.split(loc)[-1].split('.')[:-1])}", loc
    )
    mod = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(mod)
    return mod


def merge_dict(a: dict, b: dict) -> dict:
    """Merge two dictionaries together. If any keys conflict, the value in b takes precedence."""
    a.update(b)
    return a


def special_file(name: os.PathLike) -> bool:
    """Check if file is a special file."""
    for part in os.path.split(name):
        if part.startswith("_"):
            return True

    return False


def out_name(name: os.PathLike) -> os.PathLike:
    """Output name for page."""
    sname = list(os.path.split(name))
    sname[-1] = sname[-1].replace(".jinja", ".html").replace(".md", ".html")
    return os.path.join(*sname)


def find_template(
    src_path, loc: os.PathLike, markdown: bool = False
) -> Optional[os.PathLike]:
    """Find the template for a page."""
    possible_names = ["_base.jinja", "_base.html"]
    markdown_names = ["_markdown.jinja", "_markdown.html"]

    for folder in [os.path.dirname(loc), src_path]:
        for f in os.listdir(folder):
            if markdown:
                if f in markdown_names:
                    return os.path.join(folder, f)

            if f in possible_names:
                return os.path.join(folder, f)



import os
from time import time

import rich_click as click
from rich import print
from rich.prompt import Prompt, Confirm
import toml

from .kiwis import Site


@click.group()
def cli():
    pass


@cli.command()
def init():
    loc = Prompt.ask(
        "[bright_magenta]Where would you like to create your site?", default="."
    )

    if os.path.exists(loc):
        raise FileExistsError(f"{loc} already exists.")

    name = Prompt.ask(
        "[bright_blue]What is your site's name?",
        default=(
            loc if os.path.abspath(loc) != os.path.abspath(os.getcwd()) else "My Site"
        ),
    )

    print(f"[bright_magenta]Initializing {name}...")

    # Make site directory
    os.makedirs(loc)

    # Make site toml file
    with open(os.path.join(loc, "site.toml"), "w") as f:
        toml.dump({"name": name}, f)

    # Make directories
    os.mkdir(os.path.join(loc, "src"))
    os.mkdir(os.path.join(loc, "static"))

    print("[bright_green]Initialized!")


@cli.command()
@click.argument("path", type=click.Path(file_okay=False, exists=False))
def build(path):
    print(f"[bright_magenta]Building...")

    site = Site(path)
    start = time()
    site.build("./build")

    print(f"[bright_green]Built in {round((time()-start)*1000)}ms!")


@cli.command()
@click.argument("path", type=click.Path(file_okay=False, exists=False))
def dev(path):
    print("[bright_magenta]Starting development server...")

    site = Site(path)
    site.dev()


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kiwis']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'Markdown>=3.3.7,<4.0.0',
 'pymdown-extensions>=9.5,<10.0',
 'python-frontmatter>=1.0.0,<2.0.0',
 'rich-click>=1.5.1,<2.0.0',
 'rich>=12.5.1,<13.0.0',
 'toml>=0.10.2,<0.11.0',
 'watchdog>=2.1.9,<3.0.0']

entry_points = \
{'console_scripts': ['kiwis = kiwis.__main__:cli']}

setup_kwargs = {
    'name': 'kiwis',
    'version': '0.1.0b1',
    'description': 'A simple yet powerful static site generator',
    'long_description': '',
    'author': 'wxllow',
    'author_email': 'willow@wxllow.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/wxllow/kiwis',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
